use sea_orm::sea_query::Table;
use sea_orm_migration::prelude::*;
use sea_orm_migration::{MigrationName, MigrationTrait};

pub struct Migration;

impl MigrationName for Migration {
    fn name(&self) -> &str {
        "m20220804_000002_create_user_table"
    }
}

#[async_trait::async_trait]
impl MigrationTrait for Migration {
    async fn up(&self, manager: &sea_orm_migration::SchemaManager) -> Result<(), sea_orm::DbErr> {
        manager
            .create_table(
                Table::create()
                    .table(User::Table)
                    .col(
                        ColumnDef::new(User::Id)
                            .integer()
                            .primary_key()
                            .auto_increment()
                            .not_null(),
                    )
                    .col(ColumnDef::new(User::CreatedAt).date_time().not_null())
                    .col(ColumnDef::new(User::UpdatedAt).date_time().not_null())
                    .col(ColumnDef::new(User::DeletedAt).date_time().not_null())
                    .col(ColumnDef::new(User::Name).string().unique_key().not_null())
                    .col(
                        ColumnDef::new(User::Enabled)
                            .boolean()
                            .default(false)
                            .not_null(),
                    )
                    .to_owned(),
            )
            .await?;
        manager
            .create_index(
                Index::create()
                    .name("IDX_username")
                    .table(User::Table)
                    .col(User::Name)
                    .to_owned(),
            )
            .await?;

        manager
            .create_table(
                Table::create()
                    .table(UserPassword::Table)
                    .col(
                        ColumnDef::new(User::Id)
                            .integer()
                            .primary_key()
                            .auto_increment()
                            .not_null(),
                    )
                    .col(
                        ColumnDef::new(UserPassword::CreatedAt)
                            .date_time()
                            .not_null(),
                    )
                    .col(ColumnDef::new(UserPassword::UserId).integer().not_null())
                    .col(ColumnDef::new(UserPassword::Password).string().not_null())
                    .foreign_key(
                        ForeignKey::create()
                            .name("FK_user_password")
                            .to(User::Table, User::Id)
                            .from(UserPassword::Table, UserPassword::UserId)
                            .on_delete(ForeignKeyAction::Cascade)
                            .on_update(ForeignKeyAction::Cascade),
                    )
                    .to_owned(),
            )
            .await?;

        Ok(())
    }

    async fn down(&self, manager: &sea_orm_migration::SchemaManager) -> Result<(), sea_orm::DbErr> {
        manager
            .drop_table(Table::drop().table(UserPassword::Table).to_owned())
            .await?;
        manager
            .drop_table(Table::drop().table(User::Table).to_owned())
            .await?;

        Ok(())
    }
}

#[derive(Iden)]
pub enum User {
    Table,
    CreatedAt,
    UpdatedAt,
    DeletedAt,

    Id,
    Name,
    Enabled,
}

#[derive(Iden)]
pub enum UserPassword {
    Table,
    CreatedAt,

    Id,
    UserId,
    Password,
}
